@@ -5,13 +5,14 @@
 # for fixed u_h, numerically maximize p_bv (p_v) wrt correlation params; atroce car pour chaque va de param -> objfn -> auglinmodfit 
 # par contre aucune tentative de corriger les corr mat. la prevL n'est pas utilisée, elle impacte seulement u_h en input
 makeCovEst1 <- function(u_h,ZAlist,cum_n_u_h,prev_LMatrices,
-                        userLfixeds,hessUL,hessFac,w.resid,processed,phi_est,lcrandfamfam,prior.weights,family,
+                        userLfixeds,hessUL,hessFac,w.resid,processed,phi_est,lcrandfamfam,family,
                         Xpv001,v_h,auglinfixedpars
 ) {
   nrand <- length(ZAlist)
   X.Re <- processed$X.Re
   locpredictor <- processed$predictor
   next_LMatrices <- prev_LMatrices
+  if (is.null(next_LMatrices)) next_LMatrices <- list() ## NULL wrong for next_LMatrices[[rt]] <- <*M*atrix>
   Xi_cols <- attr(ZAlist,"Xi_cols")
   Lu <- u_h
   loc_lambda_est <- numeric(length(u_h))
@@ -38,7 +39,7 @@ makeCovEst1 <- function(u_h,ZAlist,cum_n_u_h,prev_LMatrices,
             diag(longLv[urange2,urange1]) <- Lcompact[jt,it]
           }
         }
-        longLv
+        Matrix(longLv) ## Matrix: 02/2106
       } ## end def makelong
       ##
       u.range <- (cum_n_u_h[rt]+1L):(cum_n_u_h[rt+1L])
@@ -67,28 +68,31 @@ makeCovEst1 <- function(u_h,ZAlist,cum_n_u_h,prev_LMatrices,
         next_LMatrices[[rt]] <- longLv
         attr(next_LMatrices[[rt]],"ranefs") <- attr(ZAlist,"ranefs")[[rt]] ## FR->FR  revoir pour matrices affectant +s termes ?
         ZALlist <- computeZAXlist(XMatrix=next_LMatrices,ZAlist=ZAlist)
-        locZAL <- post.process.ZALlist(ZALlist,predictor=locpredictor,trySparse=FALSE) ## no trySparse dans makeCovEst
-        attr(locpredictor,"ZALMatrix") <- locZAL
+        locZAL <- post.process.ZALlist(ZALlist,predictor=locpredictor,trySparse=TRUE) 
+        if (inherits(locZAL,"Matrix")) {
+          as_matrix_locZAL <- as.matrix(locZAL)
+        } else as_matrix_locZAL <- locZAL
+        if (attr(auglinfixedpars$w.resid,"unique")) attr(as_matrix_locZAL,"crossprodZAL") <- crossprod(as_matrix_locZAL)
         locTT <- cbind(Xpv001,attr(locZAL,"ZALI"))
         locw.ranefSblob <- updateW_ranefS(cum_n_u_h,processed$rand.families,lambda=loc_lambda_est,u_h,v_h) 
+        ## FR->FR auglinfixedpars is an ambiguous name since this contains $w.resid which is updated within auglinmodfit  
         auglinmodargs <- c(list(TT=locTT,ZAL=locZAL,lambda_est=loc_lambda_est,
-                                wranefblob=locw.ranefSblob),auglinfixedpars)
+                                wranefblob=locw.ranefSblob,as_matrix_ZAL=as_matrix_locZAL),auglinfixedpars)
         auglinmodblob <- do.call("auglinmodfit",auglinmodargs)
-        locd2hdv2 <- calcD2hDv2(locZAL,w.resid=auglinmodblob$w.resid,
-                                auglinmodblob$wranefblob$w.ranef) ## FR->FR ?difference avec d2hdv2 renvoyé par auglinmodfit ??
+        locd2hdv2 <- auglinmodblob$d2hdv2
         aphls <- calc.p_v(mu=auglinmodblob$muetablob$mu,u_h=auglinmodblob$u_h,
                           dvdu=auglinmodblob$wranefblob$dvdu,
                           lambda_est=loc_lambda_est,phi_est=phi_est,
                           d2hdv2=locd2hdv2,cum_n_u_h=cum_n_u_h,
                           lcrandfamfam=lcrandfamfam,processed=processed,
-                          family=family,prior.weights=prior.weights,returnLad=FALSE)
+                          returnLad=FALSE)
         if (ncol(X.Re)==0L) { ## fit ML: p_bv=p_v hence d2hdpbv reduces to d2hdv2
           # le code general se reduit a 
           ladbv <- LogAbsDetWrap(- locd2hdv2,logfac=-log(2*pi))
           # coherent avec
           # library(lme4)
           # data(sleepstudy)
-          # dat <- sleepstudy[ (sleepstudy$Days %in% 0:4) &amp; (sleepstudy$Subject %in% 331:333) ,]
+          # dat <- sleepstudy[ (sleepstudy$Days %in% 0:4) & (sleepstudy$Subject %in% 331:333) ,]
           # colnames(dat) <- c("y", "x", "group")
           # lmer( y ~ 1 + x  +( x | group ), data = dat,REML="F") 
         } else { 
