@@ -8,13 +8,14 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
                          etaFix, ## FR->FR still uses $v_h, put perhaps reconsider
                          cum_n_u_h,psi_M,
                          muetablob, 
-                         family,prior.weights,phi_est,verbose,
+                         phi_est,verbose,
                          ranFix, ## only for error message in calc_tXinvS
                          corrPars, ## only for error message in intervalStep
                          processed,
-                         ZALtZAL=NULL
+                         ZALtZAL=NULL,
+                         as_matrix_ZAL
 ) {
-  if (inherits(eta,"Matrix")) eta <- as.matrix(eta) ## eh oui...
+  #if (inherits(eta,"Matrix")) eta <- as.matrix(eta) ## 
   ### 
   BinomialDen <- processed$BinomialDen
   conv.threshold <- processed$conv.threshold
@@ -22,6 +23,7 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
   LevenbergM <- processed$LevenbergM
   betaFirst <- processed$betaFirst ## avoids explicitly solving as an aug.lin.mod.
   LMMbool <- processed$LMMbool
+  family <- processed$family
   rand.families <- processed$rand.families
   lcrandfamfam <- attr(rand.families,"lcrandfamfam")
   y <- processed$y
@@ -58,7 +60,10 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
       ## P is P in LeeL appendix p. 4 and is P_R in MolasL p. 3307 
       ## looks like leverage computation, but the ZALI columns are considered instead of the full augmented design matrix
       ## here version 1.5.3 has an interesting signed.wAugX concept
-      Pdiag <- calc.Pdiag(ZAL=ZAL,c(w.resid,w.ranef),wAugZALI= wAugX[,-seq_len(pforpv)] )
+      wAugZALI <- wAugX[,-seq_len(pforpv)]
+      if (inherits(ZAL,"sparseMatrix")) wAugZALI <- as(wAugZALI,"sparseMatrix") 
+      ## FR->FR recode to construct wAugZALI more efficiently ?
+      Pdiag <- calc.Pdiag(ZAL=ZAL,c(w.resid,w.ranef),wAugZALI=wAugZALI)
       #
       seqn_u_h <- seq_len(cum_n_u_h[nrand+1L])
       Pdiagn <- Pdiag[1:nobs]
@@ -74,29 +79,137 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
       if (is.na(vecdi2)) {
         # K1 is K1 in LeeL appendix p. 4 and is A=-ZD in MolasL p. 3307-8 
         K1 <- ZAL %id*id% K2
-        vecdi2<-rep(0,nobs) 
-        for (i in 1:nobs) vecdi2[i] <- sum( Pdiagn * coef2 * K1[1:nobs,i] )                 
+        vecdi2 <- as.numeric( (Pdiagn * coef2) %*% K1)
       }
       # coef3 =(1/Wran)(dWran/dv_h), the thing between P and K2 in the d3 coef. See LeeL12 appendix
       ## This (dlogWran_dv_h) was computed when w.ranef was computed
       if (is.na(vecdi3)) { 
-        vecdi3<-rep(0,nobs)
-        for (i in 1:nobs) vecdi3[i] <- sum( Pdiag[nobs+seqn_u_h] * dlogWran_dv_h[seqn_u_h] * K2[seqn_u_h , i] ) ## d3 reste nul pour gaussian ranef
+        #vecdi3<-rep(0,nobs)
+        #for (i in 1:nobs) vecdi3[i] <- sum( Pdiag[nobs+seqn_u_h] * dlogWran_dv_h[seqn_u_h] * K2[seqn_u_h , i] ) ## d3 reste nul pour gaussian ranef
+        vecdi3 <- as.numeric( (Pdiag[nobs+seqn_u_h] * dlogWran_dv_h[seqn_u_h]) %*% K2)
       }
       vecdi <- vecdi1+vecdi2+vecdi3 ## k_i in MolasL; le d_i de LeeL app. p. 4
       sscaled <- vecdi /2  ## sscaled := detadmu s_i= detadmu d*dmudeta/2 =d/2 in LeeL12; or dz1 = detadmu (y*-y) = detadmu m_i=0.5 k_i dmudeta = 0.5 k_i in MolasL 
     } else sscaled <-0
-    if (any(is.infinite(sscaled))) { ## FR->FR debugging code
-      mess <- pastefrom("infinite 'sscaled'.",prefix="(!) From ") 
-      stop(mess)
-    }
-    if (any(is.nan(sscaled))) { ## FR->FR debugging code
-      mess <- pastefrom("NaN 'sscaled'.",prefix="(!) From ") 
-      stop(mess)
-    }
+    # if (any(is.infinite(sscaled))) { ## debugging code
+    #   mess <- pastefrom("infinite 'sscaled'.",prefix="(!) From ") 
+    #   stop(mess)
+    # }
     return(sscaled)
   }
   
+  simple_WLS_with_Eigen <- function() {
+    qrwAugX <- betaVQ <- betaV <- NULL
+    if (inherits(wAugX,"Matrix")) {
+      ## DEFAULT COMES LAST
+      if (.spaMM.data$options$processedQRmethod == "Matrix::qr") {
+        ## see http://cran.r-project.org/web/packages/Matrix/vignettes/Comparisons.pdf
+        qrwAugX <- Matrix::qr(wAugX) ## 
+        betaV <- Matrix::qr.coef(qrwAugX,wAugz)
+      } else if (.spaMM.data$options$processedQRmethod == "lmwithSparseQ") {
+        message("lmwithSparseQ called -- should be devel code only") ## protection...
+        betaVQ <- lmwithSparseQ(wAugX,wAugz) ## tragically slow,; cf system.time's in commented code below
+        betaV <- betaVQ$coef
+        if (FALSE) { ## confinement de code de debugage 
+          qrwAugX <- Matrix::qr(wAugX)
+          essai <- Matrix::qr.Q(qrwAugX)
+          #print(max(abs(diag(tcrossprod(essai))-diag(tcrossprod(levQ)))))
+          essai <- Matrix::qr.coef(qrwAugX,wAugz)
+          print(max(abs(essai-betaV)))
+        }
+      } else if (.spaMM.data$options$processedQRmethod == "lmwithQ_sparseZAL") {     
+        if (.spaMM.data$options$USElmwithQ) {## FALSE bc lmwithQ is tragically slow
+          betaVQ <- lmwithQ(as.matrix(wAugX),wAugz) 
+          betaV <- betaVQ$coef
+        } else {
+          qrwAugX <- QRwrap(as.matrix(wAugX),useEigen=TRUE) ## maybe do not convert to matrix ? 
+          betaV <- solveWrap.vector(qrwAugX,wAugz)
+        }
+      } else { 
+        ## lmwithQ_denseZAL but ZAL still Matrix: 
+        ## only valid case would be when ZAL was Identity. (in which cas matrix::qr may be useful *if the matrix is large*)
+        wAugX <- as.matrix(wAugX) ## because both TT and wAugX kept being Matrix              }
+      }
+    } ## end all Matrix cases 
+    ## matrix case not exclusive to Matrix case because of the latest wAugX <- as.matrix(wAugX)
+    if (inherits(wAugX,"matrix")) { 
+      ## wAugX is matrix not Matrix (lmwithQ_denseZAL), with useEigen
+      if (.spaMM.data$options$USElmwithQ) {## FALSE bc lmwithQ is 'slow' because it returns Q. 
+        # The bottleneck in fitting LMMs is gettingQ from the QR thing, here or letter, only for the leverage computation.
+        betaVQ <- lmwithQ(wAugX,wAugz) 
+        betaV <- betaVQ$coef
+      } else {
+        qrwAugX <- QRwrap(wAugX,useEigen=FALSE)
+        betaV <- solveWrap.vector(qrwAugX,wAugz)
+      }
+    }
+    return(list(qrwAugX=qrwAugX,betaV=betaV,betaVQ=betaVQ))
+  }
+
+  
+  evalgainLevM <- function() {
+    ####### new values of everything, only local to this function
+    betaV <- betaV + conv_dbetaV ## betaV may be corrected below
+    if (pforpv>0) {
+      beta_eta <- betaV[seq_len(pforpv)]
+      names(beta_eta) <- colnames(X.pv)
+      if (is.null(etaFix$v_h)) v_h <- betaV[-seq_len(pforpv)] 
+    } else {if (is.null(etaFix$v_h)) v_h <- betaV}
+    u_h <- u_h_v_h_from_v_h(v_h,rand.families=rand.families,cum_n_u_h=cum_n_u_h,lcrandfamfam=lcrandfamfam,lower.v_h,upper.v_h) 
+    #checkv_h <- attr(u_h,"v_h") ## verif is v_h was corrected 
+    #if (!is.null(checkv_h)) v_h <- checkv_h
+    if (maxit.mean > 1L && !is.null(attr(u_h,"v_h"))) { ## second test = if upper.v_h or lower.v_h non NULL
+      oldv_h <- v_h
+      v_h <- attr(u_h,"v_h")
+      if (is.null(etaFix$v_h)) betaV[(pforpv+1L):length(betaV)] <- v_h ## to be copied in old_betaV, in valid space
+      ## but conv_dbetaV unchanged for assessing convergence
+    }
+    eta <- off + drop(X.pv %*% beta_eta) + drop(ZAL %id*% v_h) ## shoud use as.matrix before calling "+"
+    ## update functions u_h,v_h
+    if (!processed$GLMMbool) {
+      wranefblob <- updateW_ranefS(cum_n_u_h,rand.families,lambda_est,u_h,v_h)
+      w.ranef <- wranefblob$w.ranef
+      dvdu <- wranefblob$dvdu
+    } ## else nothing changed since lambda_est not changed
+    muetablob <- muetafn(eta=eta,BinomialDen=BinomialDen,processed=processed) 
+    mu <- muetablob$mu ## if Bin/Pois, O(n): facteur BinomialDen dans la transfo mu -> eta ## nonstandard mu des COUNTS
+    ## update functions of v_h -> blob
+    w.resid <- calc.w.resid(muetablob$GLMweights,phi_est) ## 'weinu', must be O(n) in all cases
+    d2hdv2 <- calcD2hDv2(as_matrix_ZAL,w.resid,w.ranef) ## update d2hdv2= - t(ZAL) %*% diag(w.resid) %*% ZAL - diag(w.ranef)
+    locarglist <- list(mu=mu,u_h=u_h,dvdu=dvdu,lambda_est=lambda_est,phi_est=phi_est,
+                       d2hdv2=d2hdv2,cum_n_u_h=cum_n_u_h,lcrandfamfam=lcrandfamfam,
+                       #ZAL=ZAL, ## can provide RZAL attribute but dubious if LevenbergM presumably variable w.resid
+                       processed=processed)
+    if (HL[1]==0L) {
+      currentlik <- do.call(calc.p_v,c(locarglist,only.h=TRUE))["hlik"] ##  use h in PQL/L (> v1.0)  
+    } else currentlik <- do.call(calc.p_v,locarglist)["p_v"]
+    currentlik <- unlist(currentlik) ## a way of keeping the name of the lik
+    
+    if (innerj==1L) { ## not a good idea to use the p_v computed for all u_h initially set to zero
+      gainratio <- 1
+    } else {
+      if (currentlik==-Inf) { ## obs in binary probit with extreme eta... 
+        gainratio <- -1
+      } else {
+        summand <- conv_dbetaV*(LevenbergMstep_result$rhs+ LevenbergMstep_result$dampDpD * conv_dbetaV) 
+        ## In the summand, all terms should be positive. conv_dbetaV*rhs should be positive. However, numerical error may lead to <0 or even -Inf
+        ## Further, if there are both -Inf and +Inf elements the sum is NaN and HLfit fails.
+        summand[summand<0] <- 0
+        denomGainratio <- sum(summand)
+        gainratio <- 2*(currentlik-oldlik)/denomGainratio ## cf computation in MadsenNT04 below 3.14, but generalized for D' != I ; numerator is reversed for maximization
+      }
+    }
+    ## amusing debugging code 
+    #if (class(try(if (gainratio<0) {}))=="try-error") { ## nnote that NaN *is* numeric!
+    #  mess <- pastefrom("'try(if (gainratio<0)...) failed.",prefix="(!) From ") 
+    #  stop(mess)
+    #}
+    ## levMblob$
+    return(list(gainratio=gainratio,currentlik=currentlik,betaV=betaV,beta_eta=beta_eta,
+                v_h=v_h,u_h=u_h,eta=eta,wranefblob=wranefblob,muetablob=muetablob,w.resid=w.resid,
+                d2hdv2=d2hdv2))
+  }
+  
   ## builds box constraints either NULL or non-trivial, of length n_u_h
   lower.v_h <- rep(-Inf,cum_n_u_h[nrand+1L])
   boxConstraintsBool <- FALSE
@@ -128,7 +241,12 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
     dampingfactor <- 2
   }
   qrwAugX <- NULL 
-  w.ranef <- wranefblob$w.ranef ; dlogWran_dv_h <- wranefblob$dlogWran_dv_h ; dvdu <- wranefblob$dvdu
+  w.ranef <- wranefblob$w.ranef 
+  dlogWran_dv_h <- wranefblob$dlogWran_dv_h 
+  dvdu <- wranefblob$dvdu
+  if (processed$GLMMbool) { ## w.ranef will remain = to 1/lambda, constant in this procedure
+    Sig0 <- Sigwrapper(as_matrix_ZAL,1/w.ranef,0,ZALtZAL=ZALtZAL)
+  }
   for (innerj in 1:maxit.mean) {
     ## breaks when conv.threshold is reached
     ##################
@@ -191,7 +309,7 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
       } else { ## we already have a qr, we use it
         a <- solveWrap.vector(attr(d2hdv2,"qr"), -aa,stop.on.error=stop.on.error)
       }    
-      a <- Sig %*% ( w.resid * (ZAL %id*id% a) ) ## a(0) in LeeL12
+      a <- Sig %*% ( w.resid * (ZAL %id*% a) ) ## a(0) in LeeL12
       a <- as.numeric(a) ## incase it became a Matrix, which oddly does not fit with z1-a below...
     }         
     ## and the 'something' for a(1) is computed as follows
@@ -209,13 +327,14 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
     #     } else 
     if (betaFirst)  { ### LeeL12 top p. 6 Appendix (code non optimise, useful for checking other algorithms) 
       ## c'est bien equivalent au calcul de Augz en fonction de sscaled essaye ci dessous selon LeeL12
-      tXinvS <- calc_tXinvS(Sig,X.pv,stop.on.error,lambda_est,ranFix) ## note dependence v_h -> eta -> Sig...
+      tXinvS <- calc_tXinvS(Sig,X.pv,stop.on.error) ## note dependence v_h -> eta -> Sig...
+      if (inherits(tXinvS,"try-error")) singularSigmaMessagesStop(lambda_est=lambda_est,phi_est=phi_est,corrPars=corrPars)
       ## from a(0) to a(1) LeeL12 p. 963 col 1 l 2
       a <- as.numeric(a + Sig%*% (w.resid * sscaled)) ## in case it became a Matrix...
       rhs <-  tXinvS %*% (z1-a) ## already correct in 1.0
       qr.XtinvSX <- QRwrap(tXinvS%*%X.pv) ## looks contrived but avoids computing sqrt(Sig) (! not diag !); and XinvS%*%X.pv is small
       beta_eta <- solveWrap.vector( qr.XtinvSX , rhs ,stop.on.error=stop.on.error) 
-      v_h <- solve(-d2hdv2, (tZAL %*% ((z1 - X.pv %*% beta_eta) * w.resid)+ z2*w.ranef))        
+      v_h <- solve(-d2hdv2, ( (tZAL %*% ((z1 - X.pv %*% beta_eta) * w.resid))+ z2*w.ranef))        
       betaV <- c(beta_eta,v_h)
       conv_dbetaV <- betaV - old_betaV
     } else { ### true augmented model, whether LevenbergM or not;
@@ -230,22 +349,56 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
         ##    so we keep (eta^0,v^0) here
         Augz <- c(z1- sscaled,
                   z2+ ((1/w.ranef) * tZAL) %*% (sscaled * w.resid ))  ## 
-        #browser()
         ## z2 correction  constructed from eqs (3)(4) of LeeL12 App. p5 and consistent with eq B1 MolasL:
         ## so that row 2 of wAugX.(z1-sscaled,z2+...) = Z'W1 z1 + W2 z2 => Z W1 sscaled - W2 (...) =0 => (...)=
       }
       wAugz <- Augz*sqrt.ww        
-      if ( maxit.mean > 1 && LevenbergM) { ## default(for *G*LMM) LevenbergM
+      
+      if ( maxit.mean > 1L && LevenbergM) { ## default(for *G*LMM) LevenbergM
         LM_wAugz <- wAugz - as.matrix(wAugX %*% betaV)
         ## verif correct rhs: verif_dbetaV <- safesolve.qr.vector(qrwAugX, LM_wAugz,stop.on.error=stop.on.error)
         ## verif correct v_h probleme specifique v ~ (+/-) Gamma
         #
         ## here version 1.5.3 has an interesting signed.wAugX concept
         if (.spaMM.data$options$USEEIGEN) {
-          bla <- LevenbergMstepCallingCpp(wAugX=as.matrix(wAugX),LM_wAugz=LM_wAugz,damping=damping)
-        } else bla <- LevenbergMstep(wAugX=wAugX,LM_wAugz=LM_wAugz,damping=damping,stop.on.error=stop.on.error)
-        conv_dbetaV <- bla$dbetaV## the one that will be used for assessing convergence, 
-        betaV <- betaV + conv_dbetaV ## betaV may be corrected below
+          LevenbergMstep_result <- LevenbergMstepCallingCpp(wAugX=as.matrix(wAugX),LM_wAugz=LM_wAugz,damping=damping)
+        } else LevenbergMstep_result <- LevenbergMstep(wAugX=wAugX,LM_wAugz=LM_wAugz,damping=damping,stop.on.error=stop.on.error)
+        conv_dbetaV <- LevenbergMstep_result$dbetaV## the one that will be used for assessing convergence, 
+        levMblob <- evalgainLevM()
+        if (levMblob$gainratio<0) { ## unsuccesful step: do not update anything 
+          damping <- damping*dampingfactor
+          dampingfactor <- dampingfactor*2 
+        } else { ## update everything (always TRUE when innerj=1)
+          oldlik <- currentlik <- levMblob$currentlik
+          betaV <- levMblob$betaV
+          beta_eta <- levMblob$beta_eta
+          v_h <- levMblob$v_h
+          u_h <- levMblob$u_h
+          eta <- levMblob$eta 
+          ## update functions u_h,v_h
+          if (!processed$GLMMbool) {
+            wranefblob <- levMblob$wranefblob
+            w.ranef <- wranefblob$w.ranef
+            dlogWran_dv_h <- wranefblob$dlogWran_dv_h ## (1/Wran)(dWran/dv_h), the thing between P and K2 in the d3 coef. See LeeL12 appendix
+            dvdu <- wranefblob$dvdu
+          } ## else nothing changed since lambda_est not changed
+          muetablob <- levMblob$muetablob 
+          mu <- muetablob$mu ## if Bin/Pois, O(n): facteur BinomialDen dans la transfo mu -> eta ## nonstandard mu des COUNTS
+          dmudeta <- muetablob$dmudeta
+          ## update functions of v_h -> blob
+          w.resid <- levMblob$w.resid
+          #### update fns of v_h -> blob -> w.resid
+          if (pforpv>0) {
+            if (processed$GLMMbool) {
+              Sig <- Sig0
+              diag(Sig) <- diag(Sig) + 1/w.resid 
+            } else Sig <- Sigwrapper(as_matrix_ZAL,1/w.ranef,1/w.resid,ZALtZAL=ZALtZAL) ## update v_h -> blob$GLMweights -> w.resid -> Sig -> next beta estimate
+          }
+          d2hdv2 <- levMblob$d2hdv2
+          ## cf Madsen-Nielsen-Tingleff again, and as in levmar library by Lourakis
+          damping <- damping * max(1/3,1-(2*levMblob$gainratio-1)^3)  
+          dampingfactor <- 2
+        }
       } else { ## simple aug lin (default if LMMbool) or basic IRLS depending on maxit.mean
         ## QR appears faster than alternatives with crossprod(wAugX); see version 040213
         if (.spaMM.data$options$USEEIGEN) {
@@ -253,10 +406,10 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
             calcLikArglist <- list(mu=mu,u_h=u_h,dvdu=dvdu,lambda_est=lambda_est,phi_est=phi_est,
                                d2hdv2=d2hdv2,cum_n_u_h=cum_n_u_h,lcrandfamfam=lcrandfamfam,
                                #ZAL=ZAL, ## can provide RZAL attribute 
-                               processed=processed,family=family,prior.weights=prior.weights)
+                               processed=processed)
             
             if (HL[1]==0L) { likfn <- "hlik" } else {likfn <- "p_v"} ##  use h in PQL/L (> v1.5.59) 
-            currentlik <- do.call("calc.p_v",calcLikArglist)[likfn]
+            currentlik <- do.call(calc.p_v,calcLikArglist)[likfn]
             currentlik <- unlist(currentlik) ## a way of keeping the name of the lik
             #print(c(lambda_est[1],betaV[1:4],currentlik))  ## to locate convergence issue 
             intervalBlob <- intervalStep(old_betaV=old_betaV,wAugX=wAugX,wAugz=wAugz,
@@ -264,53 +417,12 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
                                          intervalInfo=control.HLfit$intervalInfo,corrPars=corrPars,
                                          likfn=likfn)
             betaV <- intervalBlob$betaV
-          } else { ## default for LMMs
-            if (inherits(wAugX,"Matrix")) {
-              ## DEFAULT COMES LAST
-              if (.spaMM.data$options$processedQRmethod == "Matrix::qr") {
-                ## see http://cran.r-project.org/web/packages/Matrix/vignettes/Comparisons.pdf
-                qrwAugX <- Matrix::qr(wAugX) ## 
-                betaV <- Matrix::qr.coef(qrwAugX,wAugz)
-              } else if (.spaMM.data$options$processedQRmethod == "lmwithSparseQ") {
-                message("lmwithSparseQ called -- should be devel code only") ## protection...
-                betaVQ <- lmwithSparseQ(wAugX,wAugz) ## tragically slow,; cf system.time's in commented code below
-                betaV <- betaVQ$coef
-                pivI <- sort.list(betaVQ$P) ## no pivoting with lmwithQ, pivoting with sparse
-                if (FALSE) { ## confinement de code de debugage 
-                  qrwAugX <- Matrix::qr(wAugX)
-                  essai <- Matrix::qr.Q(qrwAugX)
-                  #print(max(abs(diag(tcrossprod(essai))-diag(tcrossprod(levQ)))))
-                  essai <- Matrix::qr.coef(qrwAugX,wAugz)
-                  print(max(abs(essai-betaV)))
-                }
-              } else if (.spaMM.data$options$processedQRmethod == "lmwithQ_sparseZAL") {     
-                if (.spaMM.data$options$USElmwithQ) {## FALSE bc lmwithQ is tragically slow
-                  betaVQ <- lmwithQ(as.matrix(wAugX),wAugz) 
-                  betaV <- betaVQ$coef
-                } else {
-                  qrwAugX <- QRwrap(as.matrix(wAugX),useEigen=TRUE) ## maybe do not convert to matrix ? 
-                  betaV <- solveWrap.vector(qrwAugX,wAugz)
-                }
-              } else { 
-                ## lmwithQ_denseZAL but ZAL still Matrix: 
-                ## only valid case would be when ZAL was Identity. (in which cas matrix::qr may be useful *if the matrix is large*)
-                wAugX <- as.matrix(wAugX) ## because both TT and wAugX kept being Matrix              }
-              }
-            } ## end all Matrix cases 
-            ## matrix case not exclusive to Matrix case because of the latest subcase above 
-            if (is.matrix(wAugX)) { 
-              ## wAugX is matrix not Matrix (lmwithQ_denseZAL), with useEigen
-              if (.spaMM.data$options$USElmwithQ) {## FALSE bc lmwithQ is 'slow' because it returns Q. 
-                # The bottleneck in fitting LMMs is gettingQ from the QR thing, here or letter, only for the leverage computation.
-                betaVQ <- lmwithQ(wAugX,wAugz) 
-                betaV <- betaVQ$coef
-              } else {
-                qrwAugX <- QRwrap(wAugX,useEigen=FALSE)
-                betaV <- solveWrap.vector(qrwAugX,wAugz)
-              }
-            }
+          } else {     ## default for LMMs
+            WLS_blob <- simple_WLS_with_Eigen()
+            qrwAugX <- WLS_blob$qrwAugX
+            betaV <- WLS_blob$betaV
+            betaVQ <- WLS_blob$betaVQ
           }
-          #if (any(is.nan(betaV))) browser()
         } else { ## basic IRLS without use eigen (lmwithQ_denseZAL)
           qrwAugX <- QRwrap(wAugX,useEigen=TRUE)
           betaV <- solveWrap.vector(qrwAugX, wAugz,stop.on.error=stop.on.error) ## qr.coef(qrwAugX, wAugz) ## vector
@@ -318,104 +430,45 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
         if (inherits(betaV,"try-error")) betaV <- ginv(wAugX)%*% wAugz ## occurred with large lambda either as 'init.HLfit', or by the iterative algo
         betaV <- as.numeric(betaV) #LevenbergM produces numeric/matrix...  but not necess LevenbergM here!
         if (maxit.mean > 1L) conv_dbetaV <- betaV - old_betaV
-      } ## endif LevenbergM else...
-      if (pforpv>0) {
-        beta_eta <- betaV[seq_len(pforpv)]
-        names(beta_eta) <- colnames(X.pv)
-        if (is.null(etaFix$v_h)) v_h <- betaV[-seq_len(pforpv)] 
-      } else {if (is.null(etaFix$v_h)) v_h <- betaV}
-    } ## end true augmented model       
-    u_h <- u_h_v_h_from_v_h(v_h,rand.families=rand.families,cum_n_u_h=cum_n_u_h,lcrandfamfam=lcrandfamfam,lower.v_h,upper.v_h) 
-    #checkv_h <- attr(u_h,"v_h") ## verif is v_h was corrected 
-    #if (!is.null(checkv_h)) v_h <- checkv_h
-    if (maxit.mean > 1L && !is.null(attr(u_h,"v_h"))) { ## second test = if upper.v_h or lower.v_h non NULL
-      oldv_h <- v_h
-      v_h <- attr(u_h,"v_h")
-      if (is.null(etaFix$v_h)) betaV[(pforpv+1L):length(betaV)] <- v_h ## to be copied in old_betaV, in valid space
-      ## but conv_dbetaV unchanged for assessing convergence
-    }
-    # print(paste(innerj," ",paste(beta_eta,collapse=" ")),quote=F)
-    ####### new values of everything, only tentative if LevenbergM  
-    eta <- as.matrix(off + X.pv %*% beta_eta + ZAL %id*id% v_h) ## updated at each inner iteration
-    ## update functions u_h,v_h
-    keep_wranefblob <- wranefblob ####################### 
-    wranefblob <- updateW_ranefS(cum_n_u_h,rand.families,lambda_est,u_h,v_h)
-    w.ranef <- wranefblob$w.ranef
-    dlogWran_dv_h <- wranefblob$dlogWran_dv_h ## (1/Wran)(dWran/dv_h), the thing between P and K2 in the d3 coef. See LeeL12 appendix
-    dvdu <- wranefblob$dvdu
-    keep_muetablob <- muetablob ############################ straight from function argument
-    muetablob <- muetafn(family=family,eta=eta,BinomialDen=BinomialDen,priorWeights=prior.weights) 
-    mu <- muetablob$mu ## if Bin/Pois, O(n): facteur BinomialDen dans la transfo mu -> eta ## nonstandard mu des COUNTS
-    dmudeta <- muetablob$dmudeta
-    ## update functions of v_h -> blob
-    w.resid <- calc.w.resid(muetablob$GLMweights,phi_est) ## 'weinu', must be O(n) in all cases
-    #### update fns of v_h -> blob -> w.resid
-    if ( LevenbergM ) { ## for LMM, d2hdv2 is constant // mu
-      if (pforpv>0) keep_Sig <- Sig
-      keep_d2hdv2 <- d2hdv2
-    }
-    if (pforpv>0) {
-      Sig <- Sigwrapper(ZAL,1/w.ranef,1/w.resid,ZALtZAL=ZALtZAL) ## update v_h -> blob$GLMweights -> w.resid -> Sig -> next beta estimate
-    }
-    d2hdv2 <- calcD2hDv2(ZAL,w.resid,w.ranef) ## update d2hdv2= - t(ZAL) %*% diag(w.resid) %*% ZAL - diag(w.ranef)
-    ######### for LevenbergM, we check there was a progress, and restore everything otherwise
-    if (LevenbergM) {     
-      locarglist <- list(mu=mu,u_h=u_h,dvdu=dvdu,lambda_est=lambda_est,phi_est=phi_est,
-                         d2hdv2=d2hdv2,cum_n_u_h=cum_n_u_h,lcrandfamfam=lcrandfamfam,
-                         #ZAL=ZAL, ## can provide RZAL attribute but dubious if LevenbergM presumably variable w.resid
-                         processed=processed,family=family,prior.weights=prior.weights)
-      if (HL[1]==0L) {
-        currentlik <- do.call("calc.p_v",c(locarglist,only.h=TRUE))["hlik"] ##  use h in PQL/L (> v1.0)  
-      } else currentlik <- do.call("calc.p_v",locarglist)["p_v"]
-      currentlik <- unlist(currentlik) ## a way of keeping the name of the lik
-      
-      if (innerj==1L) { ## not a good idea to use the p_v computed for all u_h initially set to zero
-        gainratio <- 1
-      } else {
-        if (currentlik==-Inf) { ## obs in binary probit with extreme eta... 
-          gainratio <- -1
-        } else {
-          summand <- conv_dbetaV*(bla$rhs+ bla$dampDpD * conv_dbetaV) 
-          ## In the summand, all terms should be positive. conv_dbetaV*rhs should be positive. However, numerical error may lead to <0 or even -Inf
-          ## Further, if there are both -Inf and +Inf elements the sum is NaN and HLfit fails.
-          summand[summand<0] <- 0
-          denomGainratio <- sum(summand)
-          gainratio <- 2*(currentlik-oldlik)/denomGainratio ## cf computation in MadsenNT04 below 3.14, but generalized for D' != I ; numerator is reversed for maximization
-        }
-      }
-      ## amusing debugging code 
-      #if (class(try(if (gainratio<0) {}))=="try-error") { ## nnote that NaN *is* numeric!
-      #  mess <- pastefrom("'try(if (gainratio<0)...) failed.",prefix="(!) From ") 
-      #  stop(mess)
-      #}
-      if (gainratio<0) { ## restore everything
-        currentlik <- oldlik ## important to restore this for the next test !
-        betaV <- old_betaV ## old_betaV should be in valid space  hence no need to check it
         if (pforpv>0) {
           beta_eta <- betaV[seq_len(pforpv)]
           names(beta_eta) <- colnames(X.pv)
-          v_h <- betaV[-seq_len(pforpv)] 
-        } else v_h <- betaV
-        u_h <- u_h_v_h_from_v_h(v_h,rand.families=rand.families,cum_n_u_h=cum_n_u_h,lcrandfamfam=lcrandfamfam,lower.v_h=NULL,upper.v_h=NULL) 
+          if (is.null(etaFix$v_h)) v_h <- betaV[-seq_len(pforpv)] 
+        } else {if (is.null(etaFix$v_h)) v_h <- betaV}
+        u_h <- u_h_v_h_from_v_h(v_h,rand.families=rand.families,cum_n_u_h=cum_n_u_h,lcrandfamfam=lcrandfamfam,lower.v_h,upper.v_h) 
         #checkv_h <- attr(u_h,"v_h") ## verif is v_h was corrected 
         #if (!is.null(checkv_h)) v_h <- checkv_h
-        #if (!is.null(attr(u_h,"v_h"))) v_h <- attr(u_h,"v_h")
-        eta <- as.matrix(off + X.pv %*% beta_eta + ZAL %id*id% v_h) ## updated at each inner iteration
-        wranefblob <- keep_wranefblob ########################## restoration
-        w.ranef <- wranefblob$w.ranef ; dlogWran_dv_h <- wranefblob$dlogWran_dv_h ; dvdu <- wranefblob$dvdu
-        muetablob <- keep_muetablob ######################################### restoration
-        mu <- muetablob$mu ; dmudeta <- muetablob$dmudeta ; w.resid <-as.vector(muetablob$GLMweights /phi_est) 
-        if (pforpv>0) Sig <- keep_Sig ###### more restoration
-        d2hdv2 <- keep_d2hdv2 ###### more restoration
-        damping <- damping*dampingfactor
-        dampingfactor <- dampingfactor*2 
-      } else { ## cf Madsen-Nielsen-Tingleff again, and as in levmar library by Lourakis
-        oldlik <- currentlik
-        damping <- damping * max(1/3,1-(2*gainratio-1)^3)  
-        dampingfactor <- 2
-      }
-    }
-    #if(length(beta_eta)>0) browser()
+        if (maxit.mean > 1L && !is.null(attr(u_h,"v_h"))) { ## second test = if upper.v_h or lower.v_h non NULL
+          oldv_h <- v_h
+          v_h <- attr(u_h,"v_h")
+          if (is.null(etaFix$v_h)) betaV[(pforpv+1L):length(betaV)] <- v_h ## to be copied in old_betaV, in valid space
+          ## but conv_dbetaV unchanged for assessing convergence
+        }
+        eta <- off + drop(X.pv %*% beta_eta) + drop(ZAL %id*% v_h) ## shoud use as.matrix before calling "+"
+        ## update functions u_h,v_h
+        if (!processed$GLMMbool) {
+          wranefblob <- updateW_ranefS(cum_n_u_h,rand.families,lambda_est,u_h,v_h)
+          w.ranef <- wranefblob$w.ranef
+          dlogWran_dv_h <- wranefblob$dlogWran_dv_h ## (1/Wran)(dWran/dv_h), the thing between P and K2 in the d3 coef. See LeeL12 appendix
+          dvdu <- wranefblob$dvdu
+        } ## else nothing changed since lambda_est not changed
+        muetablob <- muetafn(eta=eta,BinomialDen=BinomialDen,processed=processed) 
+        mu <- muetablob$mu ## if Bin/Pois, O(n): facteur BinomialDen dans la transfo mu -> eta ## nonstandard mu des COUNTS
+        dmudeta <- muetablob$dmudeta
+        ## update functions of v_h -> blob
+        w.resid <- calc.w.resid(muetablob$GLMweights,phi_est) ## 'weinu', must be O(n) in all cases
+        #### update fns of v_h -> blob -> w.resid
+        if (pforpv>0) {
+          if (processed$GLMMbool) {
+            Sig <- Sig0
+            diag(Sig) <- diag(Sig) + 1/w.resid 
+          } else Sig <- Sigwrapper(as_matrix_ZAL,1/w.ranef,1/w.resid,ZALtZAL=ZALtZAL) ## update v_h -> blob$GLMweights -> w.resid -> Sig -> next beta estimate
+        }
+        d2hdv2 <- calcD2hDv2(as_matrix_ZAL,w.resid,w.ranef) ## update d2hdv2= - t(ZAL) %*% diag(w.resid) %*% ZAL - diag(w.ranef)
+      } ## endif LevenbergM else...
+      # print(paste(innerj," ",paste(beta_eta,collapse=" ")),quote=F)
+    } ## end true augmented model       
+
     ########## nearly done with one inner iteration
     if (verbose[["trace"]]) {
       cat(paste("Inner iteration ",innerj,sep=""))
@@ -432,7 +485,6 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
       if (mean(abs(relvariation)) < conv.threshold) break; ## FR->FR mean(abs) is not standard ?  
     }
     } ## end for (innerj in 1:maxit.mean)
-   #browser()
   ##
   ### levQ stuff taken out of the (loop) fitting algo 
   levQ <- NULL  
@@ -445,6 +497,7 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
         if (.spaMM.data$options$processedQRmethod == "Matrix::qr") {
           # levQ <- as.matrix(Matrix::qr.Q(qrwAugX)) ## better to return qrwAugX ?
         } else if (.spaMM.data$options$processedQRmethod == "lmwithSparseQ") {
+          pivI <- sort.list(betaVQ$P) ## no pivoting with lmwithQ, pivoting with sparse
           levQ <- as.matrix(betaVQ$Q_ap[,pivI][,seq_len(ncol(wAugX))]) # using Q_ap, not Q
         } else if (.spaMM.data$options$processedQRmethod == "lmwithQ_sparseZAL") {     
           if (.spaMM.data$options$USElmwithQ) {## FALSE bc lmwithQ is tragically slow
@@ -464,7 +517,6 @@ auglinmodfit <- function(TT,ZAL,lambda_est,wranefblob,d2hdv2,w.resid,beta_eta,
         }
       }
     }
-    #if (any(is.nan(betaV))) browser()
   } else { ## basic IRLS without use eigen (lmwithQ_denseZAL)
     ## return qrwAugX
   }
